//
//
#ifndef FOURBY3_H
#define FOURBY3_H

#include "mdp.h"

class FourBy3State
{
public:
  // constructors
  FourBy3State(int a1 = 0, int a2 = 0): n1(a1), n2(a2) {};

  int operator== (FourBy3State & other) {
    return (n1 == other.n1 && n2 == other.n2);
  };

  // data
  int n1, n2;
};

istream & operator >> (istream & , FourBy3State &);
ostream & operator << (ostream & , FourBy3State &);

typedef int FourBy3Action;

class FourBy3StateIterator: public iterator<FourBy3State>
{
public:
  FourBy3StateIterator(): current(0,0) {init();};

  // iterator protocol
  virtual int init() { current.n1 = current.n2 = 0; return !(*this);};
  virtual int operator!() {return (current.n1 <= 3 && current.n2 <= 2);};
  virtual FourBy3State & operator () () { return current; };
  virtual int operator++()
  {
    current.n2++;
    if (current.n2 > 2) {
      current.n2 = 0;
      current.n1++;
    }
    return !(*this);
  }
  virtual void operator = (FourBy3State newValue) {
    current = newValue;
  }
protected:
  FourBy3State current;
};


class FourBy3Problem: public Problem<FourBy3State, FourBy3Action>
{
public:
  float gamma;

  FourBy3Problem(): gamma(1.0),
		  si(4*3),
		  pi(4*3)
  {
    buildModel();
  };

  virtual float discountFactor() { return gamma; };

  virtual unsigned int maxState() {
    return 4 * 3;
  };
  virtual unsigned int maxAction() {
    return 4;
  }

  // return an iterator that iterates over the states
  virtual iterator<FourBy3State> * stateIterator() {
    return new FourBy3StateIterator;
  }

  virtual list<SuccessorInfo<FourBy3State, FourBy3Action> *> *
  successors(FourBy3State s)
  {
    return &si[stateIndex(s)];
  };

  virtual list<PredecessorInfo<FourBy3State, FourBy3Action> *> *
  predecessors(FourBy3State s)
  {
    return &pi[stateIndex(s)];
  };

  virtual unsigned int stateIndex(FourBy3State s) {
    return s.n1 * 3 + s.n2;
  };

  virtual unsigned int actionIndex(FourBy3Action a) {
    return a;
  };

  // construct the model, fill in the following two vectors.
  void buildModel();
  //  void UpdateInverseModel(FourBy3State &, FourBy3Action &, FourBy3State &);
  vector<list<SuccessorInfo<FourBy3State, FourBy3Action> *> >  si;
  vector<list<PredecessorInfo<FourBy3State, FourBy3Action> *> > pi;
  virtual void printState(ostream & str, FourBy3State & s) {
    str << s;
  };
  virtual void printAction(ostream & str, FourBy3Action & a) {
    str << a;
  };

};

#endif
